﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Timers;
using System.Diagnostics;
using System.Reflection.Emit;
using System.Xml.Linq;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class MainForm : Form
    {
        private SerialHandler serial = new SerialHandler();
        private string[] speedItems = { "9600", "19200", "38400", "115200" };
        private System.Timers.Timer packageReceivingTimeout = new System.Timers.Timer(5);
        private System.Windows.Forms.Timer timer = new System.Windows.Forms.Timer();
        private System.Timers.Timer packageTransferTimer = new System.Timers.Timer(50);
        private FormClosingEventArgs formArgs;
        public string zhopa = "ZHOPA!!!";
        public MainForm()
        {
            InitializeComponent();
            PortSelector.Items.AddRange(SerialPort.GetPortNames());
            PortSelector.SelectedIndex = PortSelector.Items.Count - 1;

            SpeedSelector.Items.AddRange(speedItems);
            SpeedSelector.SelectedIndex = 3;

            packageTransferTimer.AutoReset = true;
            timer.Interval = 500;
            timer.Start();
            timer.Tick += Timer_Tick;
            ratePID_control.SubscribeOnEvent(serial);
            monitorVars1.subscribeOnMonitoringItemsReceived(serial);
            motors1.SubscribeAnswerReceived(serial);
            ratePID_control.bottom_label = InfoLabel;
            serial.HandShakeOccurred += Serial_HandShakeOccurred;
            serial.AnswerReceived += Serial_AnswerReceived;
        }

        private void Serial_AnswerReceived(object sender, SerialEventArgs e)
        {
            if (e.Answer == "EXITAPP")
            {
                serial.Close();
                base.OnFormClosing(formArgs);
            }
            else if (e.Answer == "TIMEOUT")
            {
                serial.Close();
                if (Connect_btn.InvokeRequired)
                {
                    Connect_btn.BeginInvoke(new Action(() =>
                    {
                        Connect_btn.Text = "Подключить";
                        Connect_btn.Enabled = true;
                    }));
                }
                else
                {
                    Connect_btn.Text = "Подключить";
                    Connect_btn.Enabled = true;
                }

                ResetTabControl();
            }
            else if (e.Answer.StartsWith("EXCEPTION"))
            {
                if (InfoLabel.InvokeRequired)
                {
                    InfoLabel.BeginInvoke(new Action(() =>
                    {
                        InfoLabel.Text = e.Answer.Substring(9);
                    }));
                }
                else
                {
                    InfoLabel.Text = e.Answer.Substring(9);
                }
            }
                
        }

        private void Serial_HandShakeOccurred(object sender, EventArgs e)
        {
            if (tabControl1.InvokeRequired)
            {
                tabControl1.BeginInvoke(new Action(() => tabControl1.Visible = true));
            }
            else
            {
                tabControl1.Visible = true;
            }

            if (Connect_btn.InvokeRequired)
            {
                Connect_btn.BeginInvoke(new Action(() =>
                {
                    Connect_btn.Text = "Отключить";
                    Connect_btn.Enabled = true;
                }));
            }
            else
            {
                Connect_btn.Text = "Отключить";
                Connect_btn.Enabled = true;
            }
            serial.getPIDs();
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            if (name_label.InvokeRequired) {
                name_label.BeginInvoke(new Action(() => name_label.Text = Telemetry.Instance.name)); // Асинхронный вызов
            }
            else
            {
                name_label.Text = Telemetry.Instance.name;
            }
        }

        private void Connect_btn_Click(object sender, EventArgs e)
        {
            InfoLabel.Text = "";
            Button b = sender as Button;
            bool a = serial.isOpen;
            if (!serial.isOpen)
            {
                string port;
                string speed;
                try
                {
                    port = PortSelector.SelectedItem.ToString();
                    speed = SpeedSelector.SelectedItem.ToString();
                }
                catch (Exception ex)
                {
                    InfoLabel.Text = "Порт не существует.";
                    return;
                }

                if (port != null)
                {
                    try
                    {
                        serial.PortName = port;
                        serial.BaudRate = int.Parse(speed);
                        serial.Open();
                    }
                    catch (Exception ex)
                    {
                        InfoLabel.Text = ex.Message;
                        return;
                    }
                    b.Enabled = false;
                }
            }
            else
            {
                tabControl1.Visible = false;
                serial.Close();
                b.Text = "Подключить";
            }
        }

        private void ResetTabControl()
        {
            if (tabControl1.InvokeRequired)
            {
                tabControl1.BeginInvoke(new Action(() =>
                {
                    tabControl1.Visible = false;
                    tabControl1.SelectedIndex = 0;
                }));
            }
            else
            {
                tabControl1.Visible = false;
                tabControl1.SelectedIndex = 0;
            }
            motors1.StopMotorsControl();
        }

        private void PortSelector_RefreshItems(object sender, MouseEventArgs e)
        {
            PortSelector.Items.Clear();
            PortSelector.Items.AddRange(SerialPort.GetPortNames());
            PortSelector.SelectedIndex = 0;
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            formArgs = e;
            if(serial.isOpen)serial.requestExit();
        }

        private void tabControl1_SelectedIndexChanged(object sender, EventArgs e)
        {
            TabControl tb = sender as TabControl;
            Debug.WriteLine(tb.SelectedTab.Name);
            switch (tb.SelectedTab.Name)
            {
                case "PIDs_tab":
                    serial.getPIDs();
                    break;
            }
            serial.stopMonitoring();
            motors1.StopMotorsControl();
        }

        private void TestQ_button_Click(object sender, EventArgs e)
        {
            Form form = new Form();
            form.Size = new Size(640, 480);
            TestQuaternion tq = new TestQuaternion();
            tq.Location = new Point(10, 10);
            form.Controls.Add(tq);
            form.Show();
        }
    }

    public class PopupForm : Form
    {
        public PopupForm(string message)
        {
            // 1. Настройки формы
            this.Text = "Уведомление";
            this.Size = new Size(300, 150);
            this.FormBorderStyle = FormBorderStyle.FixedDialog; // Стандартная рамка
            this.StartPosition = FormStartPosition.Manual;
            this.ShowInTaskbar = false;
            this.BackColor = Color.White;
            this.ControlBox = false; // Убираем стандартные кнопки
            this.MinimizeBox = false;
            this.MaximizeBox = false;

            // 2. Содержимое формы
            System.Windows.Forms.Label label = new System.Windows.Forms.Label()
            {
                Text = message,
                Dock = DockStyle.Fill,
                TextAlign = ContentAlignment.MiddleCenter,
                Font = new Font("Segoe UI", 10),
                Padding = new Padding(20)
            };

            Button closeButton = new Button()
            {
                Text = "OK",
                Size = new Size(80, 30),
                Anchor = AnchorStyles.Bottom | AnchorStyles.Right
            };
            closeButton.Click += (s, e) => this.Close();

            // 3. Добавление элементов
            this.Controls.Add(label);
            this.Controls.Add(closeButton);

            // 4. Позиционирование кнопки
            closeButton.Location = new Point(
                this.ClientSize.Width - closeButton.Width - 20,
                this.ClientSize.Height - closeButton.Height - 10);
        }

        // 5. Правильное отображение (без блокировки главной формы)
        public new void Show()
        {
            base.Show(); // Немодальное отображение
            this.Location = new Point(
                Cursor.Position.X - this.Width / 2,
                Cursor.Position.Y - this.Height / 2);
        }
    }
}
