﻿using ScottPlot;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class MonitorVars : UserControl
    {
        public SerialHandler serial { get; set; }
        public float[][] monitoringBuffer = new float[32][];

        private List<MonitorItem> monItems = new List<MonitorItem>();
        
        private System.Windows.Forms.Timer updateTimer = new System.Windows.Forms.Timer();
        private float[] monitoring = new float[32];
        private bool isSubscribed = false;
        private bool isMonitoring = true;
        private bool isAutoScale = true;

        private string processingName = "";
        private int processingIndex = -1;
        public MonitorVars()
        {
            InitializeComponent();
            initMonitoringBuffer();
            
            updateTimer.Interval = 100;
            AutoScale_check.Checked = isAutoScale;

            DeleteElement_menu.Click += (s, e) =>
            {
                if(monitorItems_lb.SelectedIndex != ListBox.NoMatches)
                {
                    processingIndex = monitorItems_lb.SelectedIndex;
                    serial.RemoveMonitoringItem(processingIndex);
                }
            };


            monitorItems_cb.Items.AddRange(Telemetry.Instance.getKeys());
            monitorItems_cb.SelectedIndex = 0;
            updateTimer.Tick += UpdateTimer_Tick;
            updateTimer.Start();
        }

        public void subscribeOnMonitoringItemsReceived(SerialHandler s)
        {
            if (isSubscribed) return;
            isSubscribed = true;
            serial = s;
            s.MonitoringItemsReceived += Serial_MonitoringItemsReceived;
            s.AnswerReceived += S_AnswerReceived;
        }

        private void S_AnswerReceived(object sender, SerialEventArgs e)
        {
            //Получили подтверждение о выполнении операции
            //вносим изменения в monItems(массив элементов для мониторинга)
            if (e.Answer == "ADD")
            {
                MonitorItem item = new MonitorItem(processingName, processingIndex);
                processingIndex++;
                if (monitorItems_lb.InvokeRequired)
                {
                    monitorItems_lb.Invoke(new Action(() => monitorItems_lb.Items.Add(processingName)));
                }
                else
                {
                    monitorItems_lb.Items.Add(processingName);
                }

                if (StartStop_btn.InvokeRequired)
                {
                    StartStop_btn.Invoke(new Action(() => StartStop_btn.Enabled = true));
                }
                else
                {
                    StartStop_btn.Enabled = true;
                }

                monItems.Add(item);
            }
            else if(e.Answer == "REMOVE")
            {
                monItems.RemoveAt(processingIndex);

                if (monitorItems_lb.InvokeRequired)
                {
                    monitorItems_lb.Invoke(new Action(() => 
                    monitorItems_lb.Items.RemoveAt(processingIndex)));
                }
                else
                {
                    monitorItems_lb.Items.RemoveAt(processingIndex);
                }
                
                if (processingIndex >= 0) processingIndex--;
            }else if(e.Answer == "REMOVEALL")
            {
                monItems.Clear();
                if (monitorItems_lb.InvokeRequired)
                {
                    monitorItems_lb.Invoke(new Action(() => monitorItems_lb.Items.Clear()));
                }
                else
                {
                    monitorItems_lb.Items.Clear();
                }
                if (StartStop_btn.InvokeRequired)
                {
                    StartStop_btn.Invoke(new Action(() =>
                    {
                        StartStop_btn.Text = "Старт";
                        StartStop_btn.Enabled = false;
                    })
                    );
                }
                else
                {
                    StartStop_btn.Text = "Старт";
                    StartStop_btn.Enabled = false;
                }
                isMonitoring = false;
            }
            else if (e.Answer == "MONITORON")
            {
                if (StartStop_btn.InvokeRequired)
                {
                    StartStop_btn.Invoke(new Action(() => StartStop_btn.Text = "Стоп"));
                }
                else StartStop_btn.Text = "Стоп";
                isMonitoring = true;
            }
            else if (e.Answer == "MONITOROFF")
            {
                if (StartStop_btn.InvokeRequired)
                {
                    StartStop_btn.Invoke(new Action(() => StartStop_btn.Text = "Старт"));
                }
                else StartStop_btn.Text = "Старт";
                isMonitoring = false;
            }
        }

        private void Serial_MonitoringItemsReceived(object sender, MonitoringEventArgs e)
        {
            //заполнение массивов данными
            for(int i = 0; i < monItems.Count; i++)
            {
                monItems[i].Insert(e.Data[i]);
            }

            monitoring = new float[monItems.Count];
            for (int i = 0; i < monItems.Count; i++)
            {
                monitoring[i] = (float)monItems[i].Last;
            }
        }

        private void UpdateTimer_Tick(object sender, EventArgs e)
        {
            //обновление трендов
            if (monItems.Count > 0)
            {
                MainFormPlot.Plot.Clear();
                for(int i = 0; i < monItems.Count; i++)
                {
                    var sig = MainFormPlot.Plot.Add.Signal(monItems[i].data);
                    sig.LegendText = monItems[i].name;
                    sig.Color = ScottPlot.Color.FromColor(monItems[i].color);
                    if (monitorItems_lb.InvokeRequired)
                    {
                        monitorItems_lb.Invoke(new Action(() => monitorItems_lb.Items[i] = monItems[i].name + "\t" + monItems[i].Last));
                    }
                    else
                    {
                        monitorItems_lb.Items[i] = monItems[i].name + "\t" + monItems[i].Last.ToString("0.000");
                    }
                }
                MainFormPlot.Plot.Legend.IsVisible = true;
                MainFormPlot.Plot.Legend.Orientation = ScottPlot.Orientation.Horizontal;
                if(isAutoScale) MainFormPlot.Plot.Axes.AutoScale();
                MainFormPlot.Refresh();
            }
        }

        void initMonitoringBuffer()
        {
            for (int i = 0; i < monitoringBuffer.Length; i++)
            {
                monitoringBuffer[i] = new float[8192];
            }
        }

        private void add_btn_Click(object sender, EventArgs e)
        {
            processingName = (string)monitorItems_cb.SelectedItem;
            serial.AddMonitoringItem(processingName);
        }

        private void StartStop_btn_Click(object sender, EventArgs e)
        {
            if(isMonitoring)
            {
                serial.stopMonitoring();
            }
            else
            {
                serial.startMonitoring();
            }
        }

        private void AutoScale_check_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if (cb.Checked) isAutoScale = true;
            else isAutoScale = false;
        }

        private void monitorItems_lb_MouseUp(object sender, MouseEventArgs e)
        {
            if(e.Button == MouseButtons.Right)
            {
                int index = monitorItems_lb.IndexFromPoint(e.Location);
                if(index != ListBox.NoMatches)
                {
                    monitorItems_lb.SelectedIndex = index;
                    ListBoxMenu.Show(monitorItems_lb, e.Location);
                }
            }
        }
    }

    public class MonitorItem
    {
        //Вспомогательный класс для хранения данных сигналов мониторинга
        public double[] data = new double[2048];
        public System.Drawing.Color color = System.Drawing.Color.Red;
        public string name = "";
        public int order = 0;
        private static int col = 0;
        private bool firstScan = true;
        public int Length
        {
            get { return data.Length; }
        }

        public double Last
        {
            get { return data[data.Length - 1]; }
        }

        private System.Drawing.Color[] colors =
        {
            System.Drawing.Color.Red,
            System.Drawing.Color.Green,
            System.Drawing.Color.Blue,
            System.Drawing.Color.Orange,
            System.Drawing.Color.Orchid,
            System.Drawing.Color.Purple,
            System.Drawing.Color.Pink,
            System.Drawing.Color.Black,
            System.Drawing.Color.Gray,
            System.Drawing.Color.SkyBlue,
            System.Drawing.Color.Magenta,
            System.Drawing.Color.DarkGreen,
            System.Drawing.Color.LightGreen,
            System.Drawing.Color.YellowGreen,
            System.Drawing.Color.DarkRed,
            System.Drawing.Color.Olive
        };

        private void init()
        {
            color = colors[col % 16];
            col++;
        }
        public MonitorItem()
        {
            init();
        }

        public MonitorItem(string name, int order)
        {
            this.name = name;
            this.order = order;
            init();
        }

        public void Insert(double d)
        {
            if (firstScan)
            {
                for(int i = 0; i < data.Length; i++)
                {
                    data[i] = d;
                }
                firstScan = false;
                return;
            }
            Buffer.BlockCopy(data, sizeof(double), data, 0, (data.Length - 1) * sizeof(double));
            data[data.Length - 1] = d;
        }
    }
}
