﻿using ScottPlot;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlTypes;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class Motors : UserControl
    {
        private float motorsMasterSP = 0f;
        private SerialHandler serial { get; set; }
        private System.Windows.Forms.Timer updateTimer = new System.Windows.Forms.Timer();
        private List<MonitorItem> monItems = new List<MonitorItem>();
        private bool requestNewData = false;
        public Motors()
        {
            InitializeComponent();
            for(int i = 0; i < 8; i++)
            {
                monItems.Add(new MonitorItem());
            }
        }

        private void Serial_AnswerReceived(object sender, SerialEventArgs e)
        {
            if(e.Answer == "MOTORSACT")
            {
                Debug.WriteLine(string.Join(", ", Telemetry.Instance.motor_act));
                for (int i = 0; i < monItems.Count; i++)
                {
                    monItems[i].Insert(Telemetry.Instance.motor_act[i]);
                }

                if (requestNewData)
                {
                    if (serial.isOpen) serial.sendMotorsControl();
                }
            }
        }

        public void SubscribeAnswerReceived(SerialHandler serial)
        {
            this.serial = serial;
            this.serial.AnswerReceived += Serial_AnswerReceived;
            updateTimer.Tick += UpdateTimer_Tick;
            updateTimer.Interval = 100;
        }

        private void UpdateTimer_Tick(object sender, EventArgs e)
        {
            if (monItems.Count > 0)
            {
                MotorsPlotForm.Plot.Clear();
                for (int i = 0; i < monItems.Count; i++)
                {
                    var sig = MotorsPlotForm.Plot.Add.Signal(monItems[i].data);
                    sig.LegendText = monItems[i].name;
                    sig.Color = ScottPlot.Color.FromColor(monItems[i].color);
                }
                MotorsPlotForm.Plot.Legend.IsVisible = true;
                MotorsPlotForm.Plot.Legend.Orientation = ScottPlot.Orientation.Horizontal;
                MotorsPlotForm.Plot.Axes.AutoScale();
                MotorsPlotForm.Refresh();

            }
        }

        private void MotorMSP_track_Scroll(object sender, EventArgs e)
        {
            TrackBar tb = sender as TrackBar;
            motorsMasterSP = tb.Value/100000f;
            if (tb.Value == 0) motorsMasterSP = 0f;
            Debug.WriteLine(motorsMasterSP);
            if(motorsMasterSP > 0)
            {
                float[] floats = new float[8];
                for(int i = 0; i < floats.Length; i++)
                {
                    floats[i] = motorsMasterSP;
                }
                Telemetry.Instance.motor_sp = floats;
            }
        }

        private void EnableMotors_cb_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox cb = sender as CheckBox;
            if(cb.Checked)
            {
                updateTimer.Start();
                requestNewData = true;
                serial.sendMotorsControl();
            }
            else
            {
                requestNewData = false;
                updateTimer.Stop();
            }
        }

        public void StopMotorsControl()
        {
            updateTimer.Stop();
            if (EnableMotors_cb.InvokeRequired)
            {
                EnableMotors_cb.Invoke(new Action(() => EnableMotors_cb.Checked = false));
            }
            else
            {
                EnableMotors_cb.Checked = false;
            } 
        }

        private void Motor1SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[0] = v;
        }

        private void Motor2SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[1] = v;
        }

        private void Motor3SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[2] = v;
        }

        private void Motor4SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[3] = v;
        }

        private void Motor8SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[7] = v;
        }

        private void Motor7SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[6] = v;
        }

        private void Motor6SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[5] = v;
        }

        private void Motor5SP_track_Scroll(object sender, EventArgs e)
        {
                TrackBar tb = sender as TrackBar;
                float v = tb.Value / 100000f;
                if (tb.Value == 0) v = 0f;
                Telemetry.Instance.motor_sp[4] = v;
        }
    }
}
