﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using TelemetryIO.Models;

namespace TelemetryIO
{
    public partial class RatePID : UserControl
    {
        private float[] pid0 = new float[9];
        private float[] pid1 = new float[9];
        private float[] pid2 = new float[9];
        private float[] pid3 = new float[9];
        private float zh = 0;
        private bool isSubscribed = false;
        public SerialHandler serial { get; set; }
        public Label bottom_label { get; set; }
        private object locker = new object();

        public RatePID()
        {
            InitializeComponent();
        }

        private void Serial_AnswerReceived(object sender, SerialEventArgs e)
        {
            if (bottom_label.InvokeRequired)
            {
                bottom_label.BeginInvoke(new Action(() => bottom_label.Text = e.Answer)); // Асинхронный вызов
            }
            else
            {
                bottom_label.Text = e.Answer; ;
            }

            lock (locker)
            {
                pid0 = Telemetry.Instance.pid0;
                pid1 = Telemetry.Instance.pid1;
                pid2 = Telemetry.Instance.pid2;
                pid3 = Telemetry.Instance.pid3;
            }
            if(e.Answer == "1000")change_view(pid0_panel, pid0);
            else if (e.Answer == "1001") change_view(pid1_panel, pid1);
            else if (e.Answer == "1002") change_view(pid2_panel, pid2);
            else if (e.Answer == "1003") change_view(pid3_panel, pid3);
        }

        public void SubscribeOnEvent(SerialHandler s)
        {
            if (isSubscribed) return;
            isSubscribed = true;
            serial = s;
            serial.AnswerReceived += Serial_AnswerReceived;
        }

        private void change_view(Panel panel, float[] pid)
        {
            for (int i = 0; i < panel.Controls.Count; i++)
            {
                if (panel.Controls[i] is NumericUpDown)
                {
                    NumericUpDown c = panel.Controls[i] as NumericUpDown;
                    if (c.InvokeRequired)
                    {
                        c.BeginInvoke(new Action(() => c.Value = (decimal)pid[c.TabIndex - 1]));
                    }
                    else
                    {
                        c.Value = (decimal)pid[c.TabIndex - 1];
                    }
                }
            }
        }

        private void Save_btn_Click(object sender, EventArgs e)
        {
            
            float[] floats0 = new float[9];
            for(int i = 0; i < pid0_panel.Controls.Count; i++)
            {
                if (pid0_panel.Controls[i] is NumericUpDown)
                {
                    NumericUpDown c = pid0_panel.Controls[i] as NumericUpDown;
                    floats0[c.TabIndex - 1] = (float)c.Value;
                }
            }
            serial.sendFloats(Telemetry.PID0_ADDRESS, floats0);

            float[] floats1 = new float[9];
            for (int i = 0; i < pid1_panel.Controls.Count; i++)
            {
                if (pid1_panel.Controls[i] is NumericUpDown)
                {
                    NumericUpDown c = pid1_panel.Controls[i] as NumericUpDown;
                    floats1[c.TabIndex - 1] = (float)c.Value;
                }
            }
            serial.sendFloats(Telemetry.PID1_ADDRESS, floats1);

            float[] floats2 = new float[9];
            for (int i = 0; i < pid2_panel.Controls.Count; i++)
            {
                if (pid2_panel.Controls[i] is NumericUpDown)
                {
                    NumericUpDown c = pid2_panel.Controls[i] as NumericUpDown;
                    floats2[c.TabIndex - 1] = (float)c.Value;
                }
            }
            serial.sendFloats(Telemetry.PID2_ADDRESS, floats2);

            float[] floats3 = new float[9];
            for (int i = 0; i < pid3_panel.Controls.Count; i++)
            {
                if (pid3_panel.Controls[i] is NumericUpDown)
                {
                    NumericUpDown c = pid3_panel.Controls[i] as NumericUpDown;
                    floats3[c.TabIndex - 1] = (float)c.Value;
                }
            }
            serial.sendFloats(Telemetry.PID3_ADDRESS, floats3);
        }
    }
}
